package org.dataone.audit.server;
import java.util.Iterator;
import java.util.Set;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.xml.crypto.NodeSetData;

import org.dataone.client.CNode;
import org.dataone.service.exceptions.NotImplemented;
import org.dataone.service.exceptions.ServiceFailure;
import org.dataone.service.types.v1.Node;
import org.dataone.service.types.v1.NodeList;
import org.dataone.service.types.v1.NodeType;
import org.dataone.service.types.v1.util.NodelistUtil;


public class AuditingServer 
{
	/* Instance variables */
	final protected String cnodeURL;
	
	
	/**
	 * 
	 * @param cnodeURL
	 */
	AuditingServer(String cnodeURL) 
	{
		this.cnodeURL = cnodeURL;
	}
	
	
	void execute() throws NotImplemented, ServiceFailure
	{
		/* Gets the reference node */
		CNode referenceCNode = new CNode(this.cnodeURL);
		
		/* Builds a set of coordinating nodes without the refernce cnode */
		Set<Node> cnodes = NodelistUtil.selectNodes(referenceCNode.listNodes(), 
								NodeType.CN);
		
		for(Iterator<Node> iterator = cnodes.iterator(); iterator.hasNext();)
		{
			if(iterator.next().getBaseURL().equals(this.cnodeURL))
			{
				iterator.remove();
			}
		}
		
		/* Creates a thread pool executor */
		ThreadPoolExecutor threadPoolExecutor = 
				new ThreadPoolExecutor(cnodes.size(), cnodes.size(), 0, 
						TimeUnit.SECONDS, new PriorityBlockingQueue<Runnable>());
		
		/* Test statement */
		for(Node cnode : cnodes)
		{
			threadPoolExecutor.submit(new ReadObjectsFromCNode(
					new CNode(cnode.getBaseURL())));
			break;
		}  
	}
	
	
	
	public static void main(String[] args) throws NotImplemented, ServiceFailure
	{
		if(args.length != 1)
		{
			System.out.println("Usage: AuditingServer cnode-url");
			System.exit(-1);
		}
		
		new AuditingServer(args[0]).execute();
	}
}
