package org.dataone.audit;

import static org.junit.Assert.*;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.dataone.client.D1TypeBuilder;
import org.dataone.service.types.v1.*;
import org.dataone.service.types.v1.util.AccessUtil;
import org.junit.Test;


public class TestSystemMetadataParsing {

	@Test
	public void test() {
		SystemMetadata smd = new SystemMetadata();
		smd.setIdentifier(D1TypeBuilder.buildIdentifier("myID"));
		smd.setFormatId(D1TypeBuilder.buildFormatIdentifier("myFormatID"));
		smd.setDateUploaded(new Date());
		smd.setAuthoritativeMemberNode(D1TypeBuilder.buildNodeReference("urn:node:myNode"));
		smd.setDateSysMetadataModified(new Date());
		smd.setRightsHolder(D1TypeBuilder.buildSubject("me"));
		smd.setSerialVersion(new BigInteger("12345"));
		Replica r = new Replica();
		r.setReplicaMemberNode(D1TypeBuilder.buildNodeReference("urn:node:Xanother"));
		r.setReplicationStatus(ReplicationStatus.REQUESTED);
		r.setReplicaVerified(new Date());
		smd.addReplica(r);
		r = new Replica();
		r.setReplicationStatus(ReplicationStatus.COMPLETED);
		r.setReplicaVerified(new Date());
		r.setReplicaMemberNode(D1TypeBuilder.buildNodeReference("urn:node:ONEmore"));
		smd.addReplica(r);
		
		
		AccessPolicy ap = new AccessPolicy();
		ap.addAllow(AccessUtil.createAccessRule(
				new String[]{"you","me"},
				new Permission[]{Permission.READ, Permission.WRITE, Permission.CHANGE_PERMISSION})
				);
		ap.addAllow(AccessUtil.createAccessRule(
				new String[]{"him","her"},
				new Permission[]{Permission.WRITE, Permission.READ})
				);
		smd.setAccessPolicy(ap);
		
		ReplicationPolicy rp = new ReplicationPolicy();
		rp.setNumberReplicas(5);
		rp.setReplicationAllowed(true);
		rp.addBlockedMemberNode(D1TypeBuilder.buildNodeReference("urn:node:crazyUncleMN"));
		rp.addPreferredMemberNode(D1TypeBuilder.buildNodeReference("urn:node:cousinMN"));
		rp.addPreferredMemberNode(D1TypeBuilder.buildNodeReference("urn:node:brotherMN"));
		smd.setReplicationPolicy(rp);
		
		
//		List<String> report = getSubtypes("",smd);
//		for (String s: report) {
//			System.out.println(s);
//		}
		Map<String,String> report = APITestUtils.getD1SubtypesListing("",smd,true);
		for (String s: report.keySet()) {
			System.out.println(s + "\t" + report.get(s)); 
		}
	}
	
	
	@Test
	public void testCompareSysMeta() {
		SystemMetadata sysMeta1 = createStandardSysmeta();
		SystemMetadata sysMeta2 = createStandardSysmeta();
		
		List<String> report = AuditTools.compareSystemMetadata(sysMeta1, sysMeta2);
		
		assertEquals("If they are the same, then it should be a one-line report",report.size(),1);
		assertEquals("If they are the same, then should get 'OK'",report.get(0),"OK");
		
		AccessPolicy ap = sysMeta2.getAccessPolicy();
		AccessPolicy apNew = new AccessPolicy();
		apNew.addAllow(ap.getAllow(1));
		apNew.addAllow(ap.getAllow(0));
		sysMeta2.setAccessPolicy(apNew);
		
		report = AuditTools.compareSystemMetadata(sysMeta1, sysMeta2);
		for (String line : report) {
			System.out.println(line);
		}
		System.out.println("");
		assertEquals("If they are the same, then it should be a one-line report",report.size(),1);
		assertEquals("If they are the same, then should get 'OK'",report.get(0),"OK");
		
		
		sysMeta2.getReplicationPolicy().setNumberReplicas(2);
		report = AuditTools.compareSystemMetadata(sysMeta1, sysMeta2);
		for (String line : report) {
			System.out.println(line);
		}
		assertTrue("Should have at least 2 rows", report.size() >= 2);
		assertTrue("Should have a difference in the number of replicas",
				report.get(1).contains("NumberReplicas"));	
	}

	
	
	private SystemMetadata createStandardSysmeta() {
		SystemMetadata smd = new SystemMetadata();
		smd.setIdentifier(D1TypeBuilder.buildIdentifier("myID"));
		smd.setFormatId(D1TypeBuilder.buildFormatIdentifier("myFormatID"));
		smd.setDateUploaded(new Date());
		smd.setAuthoritativeMemberNode(D1TypeBuilder.buildNodeReference("urn:node:myNode"));
		smd.setDateSysMetadataModified(new Date());
		smd.setRightsHolder(D1TypeBuilder.buildSubject("me"));
		smd.setSerialVersion(new BigInteger("12345"));
		Replica r = new Replica();
		r.setReplicaMemberNode(D1TypeBuilder.buildNodeReference("urn:node:Xanother"));
		r.setReplicationStatus(ReplicationStatus.REQUESTED);
		r.setReplicaVerified(new Date());
		smd.addReplica(r);
		r = new Replica();
		r.setReplicationStatus(ReplicationStatus.COMPLETED);
		r.setReplicaVerified(new Date());
		r.setReplicaMemberNode(D1TypeBuilder.buildNodeReference("urn:node:ONEmore"));
		smd.addReplica(r);
		
		
		AccessPolicy ap = new AccessPolicy();
		ap.addAllow(AccessUtil.createAccessRule(
				new String[]{"you","me"},
				new Permission[]{Permission.READ, Permission.WRITE, Permission.CHANGE_PERMISSION})
				);
		ap.addAllow(AccessUtil.createAccessRule(
				new String[]{"him","her"},
				new Permission[]{Permission.WRITE, Permission.READ})
				);
		smd.setAccessPolicy(ap);
		
		ReplicationPolicy rp = new ReplicationPolicy();
		rp.setNumberReplicas(5);
		rp.setReplicationAllowed(true);
		rp.addBlockedMemberNode(D1TypeBuilder.buildNodeReference("urn:node:crazyUncleMN"));
		rp.addPreferredMemberNode(D1TypeBuilder.buildNodeReference("urn:node:cousinMN"));
		rp.addPreferredMemberNode(D1TypeBuilder.buildNodeReference("urn:node:brotherMN"));
		smd.setReplicationPolicy(rp);
		
		return smd;
	}
	
//	public List<String> getSubtypes(String path, Object o) {
//		
//		List<String> results = new LinkedList<String>();
//		if (o == null) {
//			results.add(path + " = null");
//		}
//		else {
//
//			Class cl = o.getClass();
//			if (cl.getPackage().getName().startsWith("org.dataone.service.types")) {
//				Method[] methods = o.getClass().getMethods();
//				for (Method m : methods) {
//					if ((m.getName().startsWith("get") && !m.getName().equals("getClass")
//							&& m.getParameterTypes().length == 0)
//							|| m.getName().equals("xmlValue")) {
//						try {
//							Object p = m.invoke(o, (Object[]) null);
//							List<String> subresults = getSubtypes(path + "/" + m.getName().replace("get", ""),p);
//							results.addAll(subresults);
//						} catch (IllegalArgumentException e) {
//							System.err.println(m.getName());
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						} catch (IllegalAccessException e) {
//							System.err.println(m.getName());
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						} catch (InvocationTargetException e) {
//							System.err.println(m.getName());
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						}
//					} 
//					else {
//						System.err.println("skipping " + path + "/" + m.getName());
//					}
//				}
//			} else {
//				if (o instanceof String) {
//					results.add(path + " = " + o);
//				}
//				else if (o instanceof Integer) {
//					results.add(path + " = " + ((Integer)o).toString());
//				}
//				else if (o instanceof Date) {
//					results.add(path + " = " + ((Date)o).toString());
//				}
//				else if (o instanceof BigInteger) {
//					results.add(path + " = " + ((BigInteger)o).toString());
//				}
//				else if (o instanceof Boolean) {
//					results.add(path + " = " + ((Boolean)o).toString());
//				} 
//				else if (o instanceof ArrayList) {
//					if ( !((List) o).isEmpty() ) {
//						for (int j = 0; j < ((List)o).size(); j++) {
//							results.addAll( getSubtypes(path + "/"+ j,((ArrayList) o).get(j)) );
//						}
//					} else {
//						results.add(path + " = {empty}");
//					}
//				}
//			}
//		}
//		return results;
//	}
//	
//	public LinkedHashMap<String,String> getSubtypes2(String path, Object o) {
//		
//		LinkedHashMap<String,String> results = new LinkedHashMap<String,String>();
//		if (o == null) {
//			results.put(path, "null");
//		}
//		else {
//
//			Class cl = o.getClass();
//			if (cl.getPackage().getName().startsWith("org.dataone.service.types")) {
//				Method[] methods = o.getClass().getMethods();
//				for (Method m : methods) {
//					if ((m.getName().startsWith("get") && !m.getName().equals("getClass")
//							&& m.getParameterTypes().length == 0)
//							|| m.getName().equals("xmlValue")) {
//						try {
//							Object p = m.invoke(o, (Object[]) null);
//							Map<String,String> subresults = getSubtypes2(path + "/" + m.getName().replace("get", ""),p);
//							results.putAll(subresults);
//						} catch (IllegalArgumentException e) {
//							System.err.println(m.getName());
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						} catch (IllegalAccessException e) {
//							System.err.println(m.getName());
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						} catch (InvocationTargetException e) {
//							System.err.println(m.getName());
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						}
//					} 
//					else {
//						System.err.println("skipping " + path + "/" + m.getName());
//					}
//				}
//			} else {
//				if (o instanceof String) {
//					results.put(path, (String) o);
//				}
//				else if (o instanceof Integer) {
//					results.put(path, ((Integer)o).toString());
//				}
//				else if (o instanceof Date) {
//					results.put(path, ((Date)o).toString());
//				}
//				else if (o instanceof BigInteger) {
//					results.put(path, ((BigInteger)o).toString());
//				}
//				else if (o instanceof Boolean) {
//					results.put(path, ((Boolean)o).toString());
//				} 
//				else if (o instanceof ArrayList) {
//					if ( !((List) o).isEmpty() ) {
//						for (int j = 0; j < ((List)o).size(); j++) {
//							results.putAll( getSubtypes2(path + "/"+ j,((ArrayList) o).get(j)) );
//						}
//					} else {
//						results.put(path,"{empty}");
//					}
//				}
//			}
//		}
//		return results;
//	}
	
	

}
